;+**********************************************************************
; SCCS id: %W% %U% %G% 
;***********************************************************************
; PROJECT:   Magnetic holes
;
; COMPONENT: Low B
;
; LANGUAGE:  IDL
;
; NAME:             GEOVEC
;
; CALLING_SEQUENCE: outvec=GEOVEC(time,invec,[posvector],in='GEO',out='HGRTN')
;
; PURPOSE: Transform vector timeseries between geophysical coordinate systems.
;
; DESCRIPTION: Uses GEOMAT to transform axis system, optional
;              transformation to and from solar inertial systems.
;
; ARGUMENTS:
;    intime    DBLARR(tpoints)  cdf epoch millisec
;                               This time is assumed to be TT, not UT.
;                               i.e. if it is produced by CDF_EPOCH from a date
;                               the result does not take leap seconds into account.
;                               Leap seconds are accounted for by the call to QCDF_TDB 
;                               in the call.
;  OPTIONAL:
;    invector DBLARR(tpoints,3)  vectors to be transformed in insys coord.
;                                [km/s ,if velocity]
;    posvector DBLARR(tpoints,3)  position of SC in insys coord [km]
;                      if not provided position of earth is assumed.
;
;    KEYWORDS: insys/outsys/possys  STRING  any system defined in GEOMAT
;        
;        tosolar 0/1  transform to solar inertial system by adding
;               S/C velocity calculated from posvector and earth orbital vel.
;        frsolar 0/1  transform from solar inertial system by substracting
;                      S/C velocity and earth orbital vel.
;        toearth 0/1  transform to earth inertial system by adding
;               S/C velocity calculated from posvector.
;        frearth 0/1  transform from earth inertial system by substracting
;                      S/C velocity.
;
;  RETURNS:  outvec FLTARR(3,tpoints) transformed vector array
;
; FUNCTIONS USED: MARRMUL,GPOLROT,GEOMAT
;
; RESTRICTIONS: 
; ---------------------------------------------------------------------
; Copyright (c) 1996 W.J.M.Fraenz
;
; Astronomy Unit, Queen Mary and Westfield College, London, UK.
; This software is distributed WITHOUT ANY WARRANTY, and without even the
; implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
; complete copy of this copyright message with the software.
;
; ---------------------------------------------------------------------
; HISTORY:  created   2000    by Markus Fraenz (MF) @qmw
;  05/12/2001  - use HEOMAT,MARRMULO, inpu cdf epoch
;
;-***********************************************************************


 FUNCTION GEOVEC,intime,invectorin,posvectorin,insys=insys,outsys=outsys,possys=possys, $
   tosolar=tosolar,frsolar=frsolar,toearth=toearth,frearth=frearth

  nelin=N_ELEMENTS(intime)
  nelini=nelin-1
  nelvec=N_ELEMENTS(invectorin)
  inertialset=KEYWORD_SET(tosolar) OR KEYWORD_SET(frsolar) OR $
              KEYWORD_SET(toearth) OR KEYWORD_SET(frearth)
  IF nelvec NE 3*nelin THEN MESSAGE,'Incorrect number of elements: invector'
  invector=DOUBLE(invectorin)
  IF N_ELEMENTS(posvector) GT 0 THEN posvector=DOUBLE(posvectorin)

  nelpos=N_ELEMENTS(posvector)
  IF nelpos GT 0 THEN BEGIN
   IF nelpos NE 3*nelin THEN MESSAGE,'Incorrect number of elements: posvector'
  ENDIF
  IF NOT KEYWORD_SET(outsys) THEN outsys='HGRTN' ELSE outsys=STRUPCASE(outsys)

  IF NOT KEYWORD_SET(insys) THEN insys='GSE' ELSE insys=STRUPCASE(insys)

  tdb_time=QCDF_TDB(cdfepo=intime)/(8.64d6*365.25d) ;Julian centuries
  print,tdb_time,f='(g20.15)'
  inout=HEOMAT(tdb_time,posvector,insys=insys,outsys=outsys,lon=lon)
  outvector=MARRMULO(invector,inout,/vec1)

  IF  inertialset THEN BEGIN
    ;calculate earth velocity vector in HAE:
    espeed=29.7859 ; km/sec (IAU value)
    IF insys EQ 'HAE' THEN haeout=inout $
    ELSE IF outsys EQ 'HAE' THEN haeout=MARRMULO(inout,/inv1) $      
    ELSE haeout=HEOMAT(tdb_time,insys='HAE',outsys=outsys)
   ; earth velocity vector:
    earthvel=invector*0.0
    earthvel(*,0)=COS(lon-!pi/2)*espeed
    earthvel(*,1)=SIN(lon-!pi/2)*espeed
     
    IF outsys NE 'HAE' THEN earthvel=MARRMULO(earthvel,haeout,/vec)

    ; calculate s/c velocity vector in insys
    IF nelpos GT 3 THEN BEGIN 
      velocity=invector
      dtime=(intime(1:*)-intime(0:nelin-2))/1000d
      velocity(1:*,*)=posvector(1:*,*)-posvector(0:nelin-2,*)
      FOR i=0,2 DO velocity(1:*,i)=velocity(1:*,i)/dtime ; km/s
      velocity(0,*)=velocity(1,*)
      ; rotate into outsys
      velocity=MARRMULO(velocity,inout,/vec1)
    ENDIF ELSE IF nelpos LE 3 THEN BEGIN
     IF nelpos EQ 3 THEN $
      MESSAGE,'Can not calculate s/c velocity from single vector, ignored.',/con
     velocity=0.0
    ENDIF

    ; calculate velocity in outsys relativ to insys.
    ; note that if a s/c measures a velocity flying
    ; through a medium you have to ADD the s/c velocity
    ; to get the inertial velocity.
    IF KEYWORD_SET(tosolar) THEN outvector=outvector+velocity+earthvel $
     ELSE IF KEYWORD_SET(frsolar) THEN outvector=outvector-velocity-earthvel $
     ELSE IF KEYWORD_SET(frearth) THEN outvector=outvector-velocity $
     ELSE IF KEYWORD_SET(toearth) THEN outvector=outvector+velocity 
 
  ENDIF
 
 RETURN,outvector
 END



