/**==========================================================================**/
/**                                                                          **/
/**  SOURCE FILE: getephem.c                                                  **/
/**                                                                          **/
/**      Purpose: This program interpolates a state from data in a JPL       **/
/**               ephemeris file for a given Julian date. It is designed     **/
/**               to be called as a shared object, e.g. from IDL.            **/
/**                                                                          **/
/**      Arguments:                                                          **/
/**               argv[0]  <-->  EphName  char[128]     Name of file.        **/
/**               argv[1]  <-->  JDarr    double[xsize] Julian Dates         **/
/**               argv[2]  <-->  TN       short       Target Number          **/
/**               argv[3]  <-->  FmtOpt char "PosVel" or "Position"          **/
/**               argv[4]  <-->  xsize    int          size of JDarr         **/
/**               argv[5]  <-->  arr   double[xsize,6]  return vectors       **/
/**      Return Value:     arr double[xsize,6] position and velocity         **/ 
/**                                                                          **/
/**   Programmer: David Hoffman/EG5                                          **/
/**               NASA, Johnson Space Center                                 **/
/**               Houston, TX 77058                                          **/
/**               e-mail: david.a.hoffman1@jsc.nasa.gov                      **/
/**   This version adapted from rdeph.c by M.Fraenz@qmw.ac.uk 11/10/2000     **/
/**==========================================================================**/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ephem_read.h"
#include "export.h"
#ifndef TYPES_DEFINED
#include "ephem_types.h"
#endif
/**==========================================================================**/
/**  external Variables                                                        **/
/**==========================================================================**/


                
extern FILE        *Ephemeris_File;
 

int get_ephem(argc, argv)
  int argc;
  void *argv[];
{
  IDL_STRING	*str_descr;	
  stateType  State;
  char       ephemFileName[128],posvelname[7];
  char       *tgtName;
  double     Position[3] ,  outvector[6], StartTime,Time,*Time_ptr;
  short      *Target_ptr,*step_ptr;
  int        i,k , Target;
  double     *arr_ptr,*JDarr_ptr;
  IDL_LONG   x_size;

  /* Convert command line arguments to numeric values.........................*/

  str_descr	= (IDL_STRING *) argv[0];
  strncpy(ephemFileName,str_descr[0].s,str_descr[0].slen+1);
  // printf("\n\n  EphemFile:  %s\n",ephemFileName);
  str_descr	= (IDL_STRING *) argv[3];
  strncpy(posvelname,str_descr[0].s,str_descr[0].slen+1);
  // printf("\n\n  posvelname:  %s\n",posvelname);

  JDarr_ptr   = (double*)argv[1];
  arr_ptr =     (double*)argv[5];
  x_size =  *(int*)argv[4];

  Target_ptr = (short *) argv[2];
  //printf("\n\n  Target:  %d",*Target_ptr);
  //StartTime =  *Timep;
  Target = (int) *Target_ptr;

 
  /* Print planet name...................................................*/
       
  switch (Target) {
          case  0: tgtName = "Mercury";
                   break;
          case  1: tgtName = "Venus";
                   break;
          case  2: tgtName = "Earth";
                   break;
          case  3: tgtName = "Mars";
                   break;
          case  4: tgtName = "Jupiter";
                   break;
          case  5: tgtName = "Saturn";
                   break;
          case  6: tgtName = "Uranus";
                   break;
          case  7: tgtName = "Neptune";
                   break;
          case  8: tgtName = "Pluto";
                   break;
          case  9: tgtName = "Moon";
                   break;
          case 10: tgtName = "Sun";
                   break;
          default: tgtName = "Mercury";
                   break;
  }
       
  /* printf("\n\n  Target:  %s",tgtName); */
      
 
  /* Initialize the ephemeris.................................................*/

  Initialize_Ephemeris(ephemFileName);

  for ( k=0 ; k<x_size ; k++ ){
  
  Time_ptr = JDarr_ptr+k;
  Time = *Time_ptr;

  //printf("\n      JD:  %8.2f\n",Time);

  /* Compute the desired ephemeris data.......................................*/
  /*initialize outvector */
  for ( i=0 ; i<6 ; i++ ) outvector[i]=0.0;
 
  if ( !strcmp(posvelname,"PosVel") ) 
      Interpolate_State( Time , Target , &State );
  else
      Interpolate_Position( Time , Target , Position );

  /* Print the answer.........................................................*/

  if ( !strcmp(posvelname,"PosVel") ) 
     {
       for ( i=0 ; i<3 ; i++ ) outvector[i]=State.Position[i];
       for ( i=0 ; i<3 ; i++ ) outvector[i+3]=State.Velocity[i];
     
 
       /* Print position......................................................
       
       printf("\n\n  Position (km):     [1] =  % 22.15e",State.Position[0]);
       printf("\n                     [2] =  % 22.15e",State.Position[1]);
       printf("\n                     [3] =  % 22.15e",State.Position[2]); */

       /* Print velocity......................................................

       printf("\n\n  Velocity (km/sec): [1] =  % 22.15e",State.Velocity[0]);
       printf("\n                     [2] =  % 22.15e",State.Velocity[1]);
       printf("\n                     [3] =  % 22.15e\n\n\n",State.Velocity[2]);*/
       
     }
  else
     {
       for ( i=0 ; i<3 ; i++ ) outvector[i]=Position[i];
 
  
       /* Print position......................................................
       
       printf("\n\n  Position (km):     [1] =  % 22.15e",Position[0]);
       printf("\n                     [2] =  % 22.15e",Position[1]);
       printf("\n                     [3] =  % 22.15e\n\n\n",Position[2]);*/
     }

  for ( i=0 ; i<6 ; i++ ) arr_ptr[k+i*x_size]=outvector[i];
  }
 
  /* Exit normally............................................................*/
 // printf("\n File pointer fclose: %d\n",Ephemeris_File);


  fclose(Ephemeris_File);

  return(1);
}
