;+**********************************************************************
; SCCS id: %W% %U% %G% 
;***********************************************************************
;
; NAME:
;	MARRMUL
;
; PURPOSE: Multiply array of matrices.
;	
;	
; CATEGORY:
;	Coordinate transformation (orthogonal)
;
; CALLING SEQUENCE:
;
;	Result = MARRMULO (m1,m2)
;
; DESCRIPTION: This procedure multiplies two arrays
;              of orthogonal matrices m1 m2 with each other and
;              may be used for e.g. timeseries of matrices.
;              Matrices may be inverted before multiplication.
;              Note that for orthogonal matrices inverse is given
;              by transpose.
;
; INPUTS:
;	m1: Array (m,m,n/1) of n mxm matrices or n=1
;	m2: Array (m,m,n/1) of n mxm matrices or n=1
; KEYWORDS: inv1  0/1 invert m1 before multiplying
;           inv2  0/1 invert m2 before multiplying
;           vec1  0/1 m1(n,m) is array of vectors not matrices.
;                 	
; OUTPUTS:
;	Result: Array (m,m,n) of multiplied matrices m1#m2 
;
; PROCEDURE: 
;       With only m1 supplied the inverse of m1 can be calculated.
;       If m1 and m2 are only two dimensional, the simple matrix
;       product m1#m2 is returned.
;       If m1 or m2 are two dimensional each matrix of the array
;       will be multiplied with the constant matrix.
;
; MODIFICATION HISTORY:
; 	Written by: Markus Fraenz  2000
;
;-
FUNCTION MARRMULO,m1,m2,inv1=inv1,inv2=inv2,vec1=vec1
s1=SIZE(m1)
inv1set=KEYWORD_SET(inv1)
inv2set=KEYWORD_SET(inv2)
vec1=KEYWORD_SET(vec1)
IF N_PARAMS(0) EQ 1 THEN BEGIN
  IF inv1set THEN BEGIN
   IF s1(0) EQ 2 THEN RETURN,TRANSPOSE(m1)
   nelin=s1(3)
   mout=m1
   FOR i=0l,nelin-1 DO mout(*,*,i)=TRANSPOSE(m1(*,*,i))
   RETURN,mout
  ENDIF
  RETURN,m1
ENDIF
s2=SIZE(m2)

IF vec1 THEN BEGIN ; treat m1 as vector array (m,n)
 IF s1(0) EQ 1 THEN BEGIN ; just one vector
  nelini=s2(3)-1
  mout=TRANSPOSE(REFORM(m2(0,*,*)))
  IF inv2set THEN $
   FOR i=0l,nelini DO mout(i,*)=m1#TRANSPOSE(m2(*,*,i)) $
  ELSE FOR i=0l,nelini DO mout(i,*)=m1#m2(*,*,i)
  RETURN,mout
 ENDIF
 nelini=s1(1)-1
 mout=m1
 IF s2(0) EQ 2 THEN BEGIN ; just one matrix
  IF inv2set THEN $
   FOR i=0l,nelini DO mout(i,*)=m1(i,*)#TRANSPOSE(m2) $
  ELSE FOR i=0l,nelini DO mout(i,*)=m1(i,*)#m2
  RETURN,mout
 ENDIF
 IF inv2set THEN $
   FOR i=0l,nelini DO mout(i,*)=m1(i,*)#TRANSPOSE(m2(*,*,i)) $
 ELSE FOR i=0l,nelini DO mout(i,*)=m1(i,*)#m2(*,*,i)
 RETURN,mout
ENDIF


IF s1(0) EQ 2 AND s2(0) EQ 2 THEN BEGIN
 IF inv1set AND inv2set THEN RETURN,TRANSPOSE(m1)#TRANSPOSE(m2)
 IF inv1set THEN RETURN,TRANSPOSE(m1)#m2
 IF inv2set THEN RETURN,m1#TRANSPOSE(m2)
 RETURN,m1#m2 
ENDIF



IF s1(0) EQ 2 THEN BEGIN
 mout=m2*0.0
 nelini=s2(3)-1
 IF inv1set THEN m1n=TRANSPOSE(m1) ELSE m1n=m1
 IF inv2set THEN $
  FOR i=0l,nelini DO mout(*,*,i)=m1n#TRANSPOSE(m2(*,*,i)) $
 ELSE FOR i=0l,nelini DO mout(*,*,i)=m1n#m2(*,*,i)
 RETURN,mout
ENDIF

IF s2(0) EQ 2 THEN BEGIN
 mout=m1*0.0
 nelini=s1(3)-1
 IF inv2set THEN m2n=TRANSPOSE(m2) ELSE m2n=m2
 IF inv1set THEN $
  FOR i=0l,nelini DO mout(*,*,i)=TRANSPOSE(m1(*,*,i))#m2n $
 ELSE FOR i=0l,nelini DO mout(*,*,i)=m1(*,*,i)#m2n
 RETURN,mout
ENDIF

nelsize=N_ELEMENTS(s1)
FOR i=0,nelsize-1 DO $
 IF s1(i) NE s2(i) THEN MESSAGE,'Matrix arrays incompatible'

mout=m1*0.0
nelini=s1(3)-1
IF inv1set AND inv2set THEN BEGIN
  FOR i=0l,nelini DO mout(*,*,i)=TRANSPOSE(m1(*,*,i))#TRANSPOSE(m2(*,*,i))
ENDIF ELSE IF inv1set THEN BEGIN
  FOR i=0l,nelini DO mout(*,*,i)=TRANSPOSE(m1(*,*,i))#m2(*,*,i)
ENDIF ELSE IF inv2set THEN BEGIN
  FOR i=0l,nelini DO mout(*,*,i)=m1(*,*,i)#TRANSPOSE(m2(*,*,i))
ENDIF ELSE BEGIN
  FOR i=0l,nelini DO mout(*,*,i)=m1(*,*,i)#m2(*,*,i)
ENDELSE


return,mout
end 

