;+**********************************************************************
; SCCS id: %W% %U% %G% 
;***********************************************************************
; PROJECT: Cluster UK CDHF
;
; COMPONENT: QSAS panel plot
;
; LANGUAGE:  IDL
;
; NAME:
;	DOYT50
;
; PURPOSE:
; This function calculates CDF epoch seconds from yy.dddhhmmss or yy.mmddhhmmsss value
;	
; CATEGORY:
;	Time conversion
;
; CALLING SEQUENCE:
;	epoch=QCDF_DATETOEPO (yyyear0)
;
; INPUTS:
;	yyyear0: Time to be converted; Format: DBL yy.dddhhmmss or yy.mmddhhmmsss
;		 MUST be DOUBLE
;                50--99,00-49 are interpreted as 1950-2049
;		 can be a scalar or an array
; KEYWORDS: doy  input in yy.dddhhmmss format
;
; OUTPUTS:
; 	epoch:   CDF epoch ms
;		 structure is the same as the input structure
;
; RESTRICTIONS:
;               Input time should be between 1 jan 1901 and 31 dec 2049.
;               Otherwise the output is wrong. An error message is printed,
;               but the program is not interrupted.
;               Out of range values for the doy,month,hour,min,or sec fields
;               also lead to an error message.
;               Leap seconds are not considered. Numerical precision is 1 millisec.
;               Warning: Input in the form yyyy.dddhhmmss is also accepted,
;               but leads to loss in precision.
;
; MODIFICATION HISTORY:
; 	Written by:M.Fraenz 2001
;       23/02/02 doy bug removed MF
;
;-
;
  FUNCTION QCDF_DATETOEPO,yyyear0,doy=doy
;
    sy=size(yyyear0)
    nelyear=N_ELEMENTS(yyyear0)
    IF sy(n_elements(sy)-2) NE 5 THEN $
     MESSAGE,'warning:input not DOUBLE',/CONT
    timeout=DBLARR(nelyear)

    yyyear=FIX(yyyear0+1.0d-13)
 
    IF KEYWORD_SET(doy) THEN BEGIN
     yydoy0=1000.0d*(yyyear0-yyyear)+(1.0d-10)-1d
    ENDIF ELSE BEGIN
     yydoy0=10000.0d*(yyyear0-yyyear)+(1.0d-10)
    ENDELSE
    yydoy=LONG(yydoy0)
    yyhour0=100.0d*(yydoy0-yydoy)
    yyhour=FIX(yyhour0)
    yymin0=100.0d*(yyhour0-yyhour)
    yymin=FIX(yymin0)
    yysec0=100.0d*(yymin0-yymin)
    yysec=FIX(yysec0)
    msec=FIX(1000.0d*(yysec0-yysec))
    timeout(*)=3600000d*yyhour+60000d*yymin+1000d*yysec+msec

    index=WHERE(yyyear LT 50)
    IF index(0) NE -1 THEN yyyear(index)=yyyear(index)+2000
    index=WHERE(yyyear GE 50 AND yyyear LT 100)
    IF index(0) NE -1 THEN yyyear(index)=yyyear(index)+1900

    FOR i=0l,nelyear-1l DO BEGIN
     IF KEYWORD_SET(doy) THEN BEGIN
       CDF_EPOCH,yearst,yyyear(i),1,1,/comp
       timeout(i)=timeout(i)+yearst+8.64d7*yydoy(i)
     ENDIF ELSE BEGIN
       CDF_EPOCH,dayst,yyyear(i),yydoy(i)/100,(yydoy(i) MOD 100),/comp
       timeout(i)=timeout(i)+dayst
     ENDELSE
     CDF_EPOCH,timeout(i),outy,outmo,outd,outh,outm,outs,outmsec,/break
     ;print,outy,outmo,outd,outh,outm,outs,outmsec
    ENDFOR 

    index=WHERE(timeout GT 0,nel)
    IF nel NE nelyear THEN BEGIN
     MESSAGE,'Wrong time formats for:',/inf
     index0=WHERE(timeout LE 0,nel0)
     FOR i=0,nel0-1 DO PRINT,yyyear0(index0(i))
    ENDIF

    IF nelyear EQ 1 THEN timeout=timeout(0)

    RETURN,timeout
  END
;








